from messaging_client.messaging import Messaging
from time import sleep
from threading import Thread
import sys, readline, os

NAME = input("Username> ")
ADMIN = input("Admin? t/f> ")
if ADMIN.lower() == "t":
    ADMIN = True
elif ADMIN.lower() == "f":
    ADMIN = False
else:
    raise ValueError("Use t or f")

M = Messaging(NAME, ADMIN)
#Number of messages to grab at start
START_MESSAGES = 10

def print_with_input(text):
    """
    Prints around an input prompt

    Thank you jwmullally!
    https://stackoverflow.com/questions/2082387/reading-input-from-raw-input-without-having-the-prompt-overwritten-by-other-th
    """
    sys.stdout.write('\r'+' '*(len(readline.get_line_buffer())+2)+'\r')
    print(text)
    sys.stdout.write('> ' + readline.get_line_buffer())
    sys.stdout.flush()

def print_message(content):
    print_with_input("{}: {}".format(content["n"], content["c"]))

def recieve_messages():
    delay = 1
    while True:
        message = M.poll()
        if message["new"]:
            print_message(message["content"])

        sleep(delay)

def send_messages():
    while True:
        message = input("> ")
        #Forgot where I found this \/ solution, remove the prompt after input
        sys.stdout.write("\n \033[A                             \033[A")
        sys.stdout.flush()
        M.send_message(message)

def start():
    try:
        os.system('clear')
        i = START_MESSAGES
        while i != 0:
            #Grab last x messages
            message = M.get_message(M.last_message_id-i)
            print_message(message)
            i -= 1

        #Make threads
        recieve_thread = Thread(target=recieve_messages)
        send_thread = Thread(target=send_messages)
        #Start threads
        recieve_thread.start()
        send_thread.start()
        while recieve_thread.isAlive or send_thread.isAlive:
            #Keep tabs on threads to catch errors
            recieve_thread.join(1)
            send_thread.join(1)

    except KeyboardInterrupt:
        print("Good Bye!")
        os.system("reset")
        sys.exit()

    except:
        e = sys.exc_info()[0]
        print("Unexpected error {}".format(e))
        sys.exit()

if __name__ == "__main__":
    start()


import requests, re
from messaging_client.fix_json import fix_lazy_json
from ast import literal_eval

class Messaging(object):
    """
    Python client for Aaron-OS Messaging
    """
    def __init__(self, username, is_admin):
        if is_admin:
            # Adding "{ADMIN} " to the front of your name makes you admin
            self.username = "{ADMIN} " + username
        else:
            self.username = username

        #Setup URL
        self.url = "http://aaron-os-mineandcraft12.c9.io/"
        self.get_message_url = self.url + "messaging.php"
        self.put_message_url = self.url + "messager.php"

        #Get id of last message
        self.last_message_id = 0
        self.poll()

    def _parse_response(self, response):
        fixed_plain = fix_lazy_json(response).strip()
        dict_response = literal_eval(fixed_plain)
        dict_response["l"] = int(dict_response["l"])
        return dict_response

    def send_message(self, content):
        payload = {"n":self.username, "c":content}
        #Doesn't return anything so no need to save
        requests.get(self.put_message_url, params=payload)

    def get_message(self, message_id):
        #For some reason returns message_id + 1
        message = requests.get(self.get_message_url, params={"l":message_id})
        message = self._parse_response(message.text)
        return message

    def poll(self):
        #Check for new messages
        check = self.get_message(-1)

        if int(check["l"])+1 != self.last_message_id:
            #Get latest message
            #For some reason l = l + 1
            latest = self.get_message(int(check["l"]))
            self.last_message_id = latest["l"]
            return {"new":True, "content":latest}

        return {"new":False, "content":check}


import tokenize
import token
from io import StringIO

def fix_lazy_json (in_text):
    """
    Fix lazy json(duh)

    thanks to psanchez for the solution
    https://stackoverflow.com/questions/4033633/handling-lazy-json-in-python-expecting-property-name
    """
    tokengen = tokenize.generate_tokens(StringIO(in_text).readline)

    result = []
    for tokid, tokval, _, _, _ in tokengen:
        # fix unquoted strings
        if (tokid == token.NAME):
            if tokval not in ['true', 'false', 'null', '-Infinity', 'Infinity', 'NaN']:
                tokid = token.STRING
                tokval = u'"%s"' % tokval

        # fix single-quoted strings
        elif (tokid == token.STRING):
            if tokval.startswith ("'"):
                tokval = u'"%s"' % tokval[1:-1].replace ('"', '\\"')

        # remove invalid commas
        elif (tokid == token.OP) and ((tokval == '}') or (tokval == ']')):
            if (len(result) > 0) and (result[-1][1] == ','):
                result.pop()

        # fix single-quoted strings
        elif (tokid == token.STRING):
            if tokval.startswith ("'"):
                tokval = u'"%s"' % tokval[1:-1].replace ('"', '\\"')

        result.append((tokid, tokval))

    return tokenize.untokenize(result)


from setuptools import setup

setup(
    name='AaronOS_Messaging',
    version='0.0.2',
    description='A command line client for the messaging app in Aaron OS',
    url='https://github.com/edison-moreland/AaronOS-Messaging-Client',
    download_url='https://github.com/edison-moreland/AaronOS-Messaging-Client/archive/0.0.2.tar.gz',
    author='Edison Moreland',
    author_email='edison@devdude.net',
    license='MIT',
    packages=['messaging_client'],
    install_requires=['requests'],
    entry_points={
        'console_scripts': [
            'messaging = messaging_client.main:start'
        ]
    }
    )